<?php

namespace FiloBlu\ExtInventory\Helper;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\IsProductSalableInterface;
use Magento\InventorySalesApi\Api\StockResolverInterface;
use Magento\InventorySales\Model\ResourceModel\StockIdResolver;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Store\Model\ResourceModel\Website\CollectionFactory;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Store\Api\StoreRepositoryInterface;
use Magento\Framework\App\ResourceConnection;

/**
 * Class Data
 * @package FiloBlu\ExtInventory\Helper
 */
class Data extends AbstractHelper
{
    /**
     * @var IsProductSalableInterface
     */
    protected $isProductSalableInterface;
    /**
     * @var StockResolverInterface
     */
    protected $stockResolver;
    /**
     * @var ProductRepository
     */
    protected $productRepository;
    /**
     * @var CollectionFactory
     */
    protected $websiteCollectionFactory;
    /**
     * @var StockIdResolver
     */
    protected $stockIdResolver;
    /**
     * @var StoreManagerInterface
     */
    public $storeManager;
    /**
     * @var OrderRepositoryInterface
     */
    public $orderRepositoryInterface;
    /**
     * @var StoreRepositoryInterface
     */
    protected $storeRepositoryInterface;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * Data constructor.
     * @param Context $context
     * @param IsProductSalableInterface $isProductSalableInterface
     * @param StockResolverInterface $stockResolver
     * @param CollectionFactory $websiteCollectionFactory
     * @param ProductRepository $productRepository
     * @param StockIdResolver $stockIdResolver
     * @param StoreManagerInterface $storeManager
     * @param OrderRepositoryInterface $orderRepositoryInterface
     * @param StoreRepositoryInterface $storeRepositoryInterface
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        Context $context,
        IsProductSalableInterface $isProductSalableInterface,
        StockResolverInterface $stockResolver,
        CollectionFactory $websiteCollectionFactory,
        ProductRepository $productRepository,
        StockIdResolver $stockIdResolver,
        StoreManagerInterface $storeManager,
        OrderRepositoryInterface $orderRepositoryInterface,
        StoreRepositoryInterface $storeRepositoryInterface,
        ResourceConnection $resourceConnection
    )
    {
        parent::__construct($context);
        $this->isProductSalableInterface = $isProductSalableInterface;
        $this->stockResolver = $stockResolver;
        $this->websiteCollectionFactory = $websiteCollectionFactory;
        $this->productRepository = $productRepository;
        $this->stockIdResolver = $stockIdResolver;
        $this->storeManager = $storeManager;
        $this->orderRepositoryInterface = $orderRepositoryInterface;
        $this->storeRepositoryInterface = $storeRepositoryInterface;
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @return bool
     */
    public function isEnabled(): bool
    {
        return true;
    }

    /**
     * @param $productSku
     * @param $websiteId
     * @return bool
     * @throws NoSuchEntityException
     */
    public function isSalable($productSku, $websiteId): bool
    {
        //$product = $this->getProductById($productId);
        $websiteCode = $this->getWebsiteCodeById($websiteId);
        $stockId = $this->getStockIdByWebsiteCode($websiteCode);

        if (is_object($productSku)) {
            $productSku = $productSku->getSku();
        }

        return $this->isProductSalableInterface->execute($productSku, $stockId);
    }

    /**
     * @param $websiteId
     * @return mixed
     */
    public function getWebsiteCodeById($websiteId)
    {
        $collection = $this->websiteCollectionFactory->create();
        $collection->addFieldToFilter('website_id', ['eq' => $websiteId]);
        $websiteData = $collection->getData();
        return $websiteData[0]['code'];
    }

    /**
     * @param $website_code
     * @return int|null
     * @throws NoSuchEntityException
     */
    public function getStockIdByWebsiteCode($website_code)
    {
        $stock = $this->stockResolver->execute(SalesChannelInterface::TYPE_WEBSITE, $website_code);
        return $stock->getStockId();
    }

    /**
     * @param $order
     * @return int|null
     * @throws NoSuchEntityException
     */
    public function getStockIdByOrder(OrderInterface $order){
        $websiteId = $this->storeManager->getStore($order->getStoreId())->getWebsiteId();
        $websiteCode = $this->storeManager->getWebsite($websiteId)->getCode();
        $stockId = $this->stockIdResolver->resolve(SalesChannelInterface::TYPE_WEBSITE, $websiteCode);
        return $stockId;
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    public function getCurrentStoreviewId(){
        $currentStore = $this->storeManager->getStore();
        return $currentStore->getId();
    }

    /**
     * @param $id
     * @return ProductInterface|mixed
     * @throws NoSuchEntityException
     */
    public function getProductById($id)
    {
        return $this->productRepository->getById($id);
    }

    /**
     * @param $sku
     * @return ProductInterface|Product|null
     * @throws NoSuchEntityException
     */
    public function getProductBySku($sku)
    {
        return $this->productRepository->get($sku);
    }

    /**
     * @param $order_id
     * @return int|null
     */
    public function getOrderStoreviewIdByOrderId($order_id){
        $order = $this->orderRepositoryInterface->get($order_id);
        return $order->getStoreId();
    }

    /**
     * @return array
     */
    public function getAllStoresIds(){
        $all_stores = $this->storeRepositoryInterface->getList();
        $all_stores_ids = array();
        foreach ($all_stores as $store) {
            $all_stores_ids[] = $store->getId();
        }
        return $all_stores_ids;
    }


    /**
     * @param $product
     * @param $websiteCode
     * @return mixed
     */
    public function getSalableQty($product,$websiteCode){

        /**
         * select
        Q2.qty - Q1.qty_reserved
        FROM
        (
        SELECT
        isi.sku,
        SUM(IF(so.state in('new', 'processing'), ifos.qty, 0)) as qty_reserved
        FROM
        `inventory_source_item` AS `isi`
        INNER JOIN `inventory_source_stock_link` AS `issl` ON
        issl.source_code = isi.source_code
        INNER JOIN `inventory_stock_sales_channel` AS `issc` ON
        issc.stock_id = issl.stock_id
        LEFT JOIN `inventory_fb_orderitem_source` AS `ifos` ON
        ifos.sku = isi.sku
        AND ifos .final_inventory_source_id = isi.source_code
        LEFT JOIN `sales_order` AS `so` ON
        ifos.order_id = so.entity_id
        WHERE
        (isi.sku = 'WBCE60588BARGLLPO5534')
        AND (issc.code = 'santoni_us')
        GROUP BY
        `isi`.`sku`) Q1
        Left JOIN (
        SELECT
        `isi`.`sku`,
        SUM(isi.quantity) as qty
        FROM
        `inventory_source_item` AS `isi`
        INNER JOIN `inventory_source_stock_link` AS `issl` ON
        issl.source_code = isi.source_code
        INNER JOIN `inventory_stock_sales_channel` AS `issc` ON
        issc.stock_id = issl.stock_id
        WHERE
        (isi.sku = 'WBCE60588BARGLLPO5534')
        AND (issc.code = 'santoni_us')
        GROUP BY
        `isi`.`sku`) as Q2 ON
        Q1.sku = Q2.sku
         */
        $sku = $product->getSku();
        $connection = $this->resourceConnection->getConnection();
        $q1 = $connection->select()
            ->from(['isi' => $this->resourceConnection->getTableName('inventory_source_item')],
                [
                    'isi.sku',
                    "SUM(IF(so.state in('new', 'processing'),ifos.qty,0)) AS qty"
                ])
            ->joinInner(
                ['issl' => $this->resourceConnection->getTableName('inventory_source_stock_link')],
                'issl.source_code = isi.source_code',[]
            )
            ->joinInner(
                ['issc'=>$this->resourceConnection->getTableName('inventory_stock_sales_channel')],
                'issc.stock_id = issl.stock_id',[]
            )->joinLeft(
                ['ifos'=>$this->resourceConnection->getTableName('inventory_fb_orderitem_source')],
                'ifos.sku = isi.sku AND ifos .final_inventory_source_id = isi.source_code',[]
            )->joinLeft(
                ['so'=>$this->resourceConnection->getTableName('sales_order')],
                'ifos.order_id = so.entity_id',[]
            )
            ->where('isi.sku = ?', $sku)
            ->where('issc.code = ?', $websiteCode)
            ->group('isi.sku');


        $q2 = $connection->select()
            ->from(['isi' => $this->resourceConnection->getTableName('inventory_source_item')],
                [
                    'isi.sku',
                    "SUM(isi.quantity) as qty"
                ])
            ->joinInner(
                ['issl' => $this->resourceConnection->getTableName('inventory_source_stock_link')],
                'issl.source_code = isi.source_code',[]
            )
            ->joinInner(
                ['issc'=>$this->resourceConnection->getTableName('inventory_stock_sales_channel')],
                'issc.stock_id = issl.stock_id',[]
            )
            ->where('isi.sku = ?', $sku)
            ->where('issc.code = ?', $websiteCode)
            ->group('isi.sku');
        $select = $connection->select()->from(
            ['q1'=>new \Zend_Db_Expr("($q1)")],['available'=>new \Zend_Db_Expr('q2.qty - q1.qty')]
        )->joinLeft(['q2'=> new \Zend_Db_Expr("($q2)")],
            'q1.sku = q2.sku',[]);

        $productData = $connection->query($select)->fetch();
        if($productData && isset($productData['available'])){
            return $productData['available'];
        }
    }
}